#%%
from IPython.core.pylabtools import figsize
figsize(18, 9)

#%%  Reading the challenge dataset
import matplotlib.pyplot as plt
import numpy as np
import pathlib

def readfile(f):
    with f.open() as r:
        s=r.read()
    im = np.array([[float(n) for n in line.split()] for line in s.splitlines()])
    return im

ddir = pathlib.Path("dataset")
dataset = []
assert(ddir.exists())
for c in ddir.glob("*"):
    print(c.name, flush=True)
    for f in c.glob("*.dat"):
        im = readfile(f)
        dataset.append((im,c.name))

print("Read {} images".format(len(dataset)))

#%% Display some info about the dataset (and learn to access it)
print("Total images: ",len(dataset))

indices_first5 = range(5) # the first 5 images (0 to 4 incl)
indices_random5 = np.random.choice(len(dataset), 5, replace=False) # 5 random images

for i in indices_random5:
    im = dataset[i][0]
    label = dataset[i][1]
    print("Image index {} has shape {} and label {}".format(
            i, im.shape, label))
  

#%% Displays all images with the respective labels
    
for (im,label) in dataset: # this automatically unpacks the tuple
    plt.imshow(im)
    plt.colorbar()
    plt.title(label)
    plt.show() 

#%% A supersimple framework for image classification
    
def makefeatures(im):
    """ This function takes an image as input and returns a 1D array of features """
    # This implementation is way too simple. You should make something better
    
    x = [im.shape[0],     # Number of rows in the input image
         np.mean(im),     # Mean intensity of the image
         #np.std(im)
         ]      # Std dev of the intensity of the image
    return np.array(x)


X=[]    # We'll store a list of feature vectors, one per sample
y=[]    # These will be the corresponding labels (as strings)
for (im,label) in dataset:
    X.append(makefeatures(im))
    y.append(label)
X=np.array(X)
y=np.array(y)

print(X.shape, y.shape)


#%% A little extra: visualize the feature matrix, and understand why feature
#   normalization is a good idea

import seaborn as sns
sns.heatmap(X,cmap="viridis")
#sns.heatmap((X-np.mean(X,axis=0,keepdims=True))/np.std(X,axis=0,keepdims=True),cmap="viridis")

#%%
# We have to split the dataset in a training + testing set!
perm = np.random.permutation(len(y)) # Randomizing is important, because our data is sorted by class!
tr_ix = perm[:200] # Training indices
te_ix = perm[200:] # Testing indices
assert(not np.any(np.isin(tr_ix,te_ix))) # Make sure they don't overlap!

#%% Let's define a classifier and train it

from sklearn import dummy
from sklearn import svm
from sklearn import neighbors
from sklearn import preprocessing
from sklearn import feature_extraction
from sklearn.pipeline import make_pipeline
from sklearn import metrics
from sklearn import ensemble


clf = neighbors.KNeighborsClassifier(n_neighbors = 1) # A support vector machine classifier
# clf = svm.SVC() # A support vector machine classifier
# clf = ensemble.RandomForestClassifier() # A random forest classifier

# Optionally, wrap our classifier with a pipeline that standardizes the input features
# clf=make_pipeline(
#        preprocessing.StandardScaler(),
#        clf)

# This guy always returns the most frequent class in the training data, ignoring the features
# Always compare to his performance as a baseline!!!
dummy_clf = dummy.DummyClassifier() 

# Train our classifiers
clf.fit(X[tr_ix,:], y[tr_ix])
dummy_clf.fit(X[tr_ix,:], y[tr_ix])

# Get the estimates from our classifier
yhat=clf.predict(X[te_ix,:])
dummy_yhat=dummy_clf.predict(X[te_ix,:])

print("Accuracy: {:.2f}".format(np.mean(yhat==y[te_ix])))
print("Dummy accuracy: {:.2f}".format(np.mean(dummy_yhat==y[te_ix])))

# If you want to know the probabilities of each class:
yhat_proba = clf.predict_proba(X[te_ix,:])
for i in range(5):
    print("Testing sample {} (index {}) has true class {}".format(
           i, te_ix[i], y[te_ix[i]]))
    for j in range(len(clf.classes_)):
        print("- Estimated probability for class {}:\t{}".format(
           clf.classes_[j], yhat_proba[i,j]))

#%% Homework
#
# - Does feature standardization help?
# - We implemented a lame holdout split.  Cross validation would be better when comparing features and approaches: 
#   implement cross validation yourself, or use sklearn's facilies to do that: http://scikit-learn.org/stable/modules/cross_validation.html
# - Compare a few classifiers.  Usually, sklearn.ensemble.RandomForestClassifier works really well
# - Visualize the dataset, study the different classes, and try to understand what you could use as good features for our problem
# - Use your ninja image processing skillz to compute these features (redefine makeFeatures(im))
#   You can compute as many as you want, of course.  But the dataset is small, I'd not exceed 10 features.
# - Beat all your friends on the accuracy obtained with 10-fold cross validation!

# Submit your result here: https://goo.gl/forms/zfJt08HjTK2aKgql2

#%% Hint for implementing cross validation using sklearn

import sklearn.model_selection

# Cross validation strategy
cv = sklearn.model_selection.KFold(n_splits=10, 
                                   shuffle=True,    # Important as our data is sorted by class!
                                   random_state=0)  # For reproducibility

predicted = sklearn.model_selection.cross_val_predict(clf, X, y, cv=cv)
# The i-th element is the class predicted by a classifier that was trained on the remaining 9 folds

print("Accuracy:", metrics.accuracy_score(y, predicted))
print("Should be the same:", np.mean(predicted==y))