#%% Some image processing on individual images
# References: 
# - http://www.scipy-lectures.org/packages/scikit-image
# - http://scikit-image.org/docs/dev/api/api.html


#%%
from IPython.core.pylabtools import figsize
figsize(18, 9)

#%%  Reading the challenge dataset
import matplotlib.pyplot as plt
import numpy as np
import pathlib

def readfile(f):
    with f.open() as r:
        s=r.read()
    im = np.array([[float(n) for n in line.split()] for line in s.splitlines()])
    return im

ddir = pathlib.Path("dataset")
dataset = []
assert(ddir.exists())
for c in ddir.glob("*"):
    print(c.name, flush=True)
    for f in c.glob("*.dat"):
        im = readfile(f)
        dataset.append((im,c.name))

print("Read {} images".format(len(dataset)))


#%% Histogram
import skimage
import skimage.exposure

# Let's pick an image
im,label = dataset[0]

#%%
plt.imshow(im)
plt.colorbar()

#%%
# Compute the image histogram (how many pixels with each intensity value?)
count, value = skimage.exposure.histogram(im)
plt.plot(value,count)
plt.xlabel("Intensity value")
plt.ylabel("Count of pixels")

#%%
# Or directly use the matplotlib histogram function

binedges = np.arange(0,np.max(im),0.01) # 1 cm bins
plt.hist(im.flatten(),bins=binedges)
plt.xlabel("Intensity value")
plt.ylabel("Count of pixels")

#%% Thresholding

im_t = im > 0.02
print(im_t.dtype)
plt.imshow(im_t)

#%% Labeling connected components
# Let's build an image with two connected components
im_t2 = np.vstack((im_t,np.fliplr(im_t)))
plt.imshow(im_t2)
plt.colorbar()

#%% ... alternatively use a threshold that generates many
im_t2 = im > 0.08
print(im_t2.dtype)
plt.imshow(im_t2)

#%%
import skimage.measure
components = skimage.measure.label(im_t2)
print(components.dtype)
plt.imshow(components, cmap="Set1")
plt.colorbar()

#%% Display the first few components
fig,axs = plt.subplots(nrows=3,ncols=3,figsize=(15,10))
for i,ax in enumerate(axs.flatten()):
    ax.imshow(components == i)
    ax.set_title("Label: {}".format(i))

#%% Properties
rps = skimage.measure.regionprops(components)
# What's inside? http://scikit-image.org/docs/dev/api/skimage.measure.html#skimage.measure.regionprops

print("We have {} labels (in addition to label 0, i.e. background)".format(np.max(components)))
print("We have {} regionprops objects".format(len(rps)))

for rp in rps:
    print("Region label {} has perimeter {}, area {}, eccentricity {}".format(
            rp.label, rp.perimeter, rp.area, rp.eccentricity))
    plt.imshow(rp.image, vmin = 0, vmax = 1)
    plt.show()



#%% Canny on a gray scale image
import skimage.feature
plt.imshow(im)
plt.show()

edges = skimage.feature.canny(im, sigma = 1)
plt.imshow(edges)

#%%  Effect of sigma (better to try that on a more meaningful image)

sigmas = [0.1, 0.5, 1, 5]
fig,axs = plt.subplots(ncols=len(sigmas),figsize=(25,10))
for sigma,ax in zip(sigmas,axs):
    edges = skimage.feature.canny(im, sigma = sigma)
    ax.imshow(edges)
    ax.set_title("Sigma: {}".format(sigma))



#%% Median filtering
import skimage.filters
import skimage.morphology
plt.imshow(skimage.filters.rank.median(
        im,
        skimage.morphology.disk(5)))
                                  
#%% Morphology (on grayscale images)

plt.imshow(skimage.morphology.erosion(
        im,
        skimage.morphology.disk(5)))
plt.show()

plt.imshow(skimage.morphology.dilation(
        im,
        skimage.morphology.disk(5)))
plt.show()

plt.imshow(skimage.morphology.opening(
        im,
        skimage.morphology.disk(5)))
plt.show()

plt.imshow(skimage.morphology.closure(
        im,
        skimage.morphology.disk(5)))
plt.show()